/*
 * Copyright (C) 2017 Fairphone
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.fairphone.fplauncher3.edgeswipe;

import java.util.ArrayList;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.content.SharedPreferences;

import com.android.launcher3.AppInfo;
import com.android.launcher3.compat.LauncherActivityInfoCompat;
import com.android.launcher3.IconCache;
import com.android.launcher3.ItemInfo;
import com.android.launcher3.Launcher;
import com.android.launcher3.LauncherAppState;
import com.android.launcher3.LauncherModel;
import com.android.launcher3.LauncherProvider;
import com.android.launcher3.LauncherSettings;
import com.android.launcher3.ShortcutInfo;

import com.fairphone.fplauncher3.edgeswipe.editor.FavoritesStorageHelper;

/**
 * Migration tool from original FairphoneLauncher3 to newer version completely
 * based on Launcher 3.
 *
 * Previous versions of FairphoneLauncher3 included an edgeswipe menu instead
 * of the hotseat. New FairphoneLauncher3 does not include edgeswipe anymore.
 * This class provides functionality to copy items from edgeswipe menu to
 * hotseat in order to provide users access to their favorites after migration.
 */
public abstract class FavoritesToHotseatConverter {
    private static final String FAVORITES_MIGRATED_KEY = "EDGESWIPE_FAVORITES_MIGRATED_KEY";

    /**
     * Indicates whether migration of Edgeswipe favorites to Hotseat items
     * should be performed.
     */
    public static boolean hasMigrationRunAlready(Launcher launcher) {
        return launcher == null ||
            getSharedPrefs(launcher).getBoolean(FAVORITES_MIGRATED_KEY, false);
    }

    /**
     * Copy favorites from edgeswipe menu to hotseat (unless hotseat has icons
     * already or database has been newly generated).
     *
     * In any case: Removes legacy settings for edgeswipe favorites and sets a
     * flag to prevent the migration from running in the future.
     */
    public static void migrateFavoritesToHotseat(Launcher launcher) {
        // Skip migration if hotseat contains icons or DB was newly created.
        if (!wouldOverwriteHotseat(launcher)) {

            ArrayList<ComponentName> componentNames =
                FavoritesStorageHelper.getComponentNamesOfFavorites(launcher);

            for (int i = 0; i < componentNames.size(); i++) {
                ComponentName componentName = componentNames.get(i);

                if (componentName == null) {
                    continue;
                }

                ItemInfo info = hotseatShortcutFromComponentName(launcher,
                            componentName, i);

                if (info == null) {
                    continue;
                }

                LauncherModel.addItemToDatabase(launcher, info,
                        LauncherSettings.Favorites.CONTAINER_HOTSEAT,
                        info.screenId, info.cellX, info.cellY);
            }
            reloadWorkspace();
        }
        setMigrationDone(launcher);
    }

    private static boolean wouldOverwriteHotseat(Launcher launcher) {
        return launcher.getHotseat().hasIcons();
    }

    private static ShortcutInfo hotseatShortcutFromComponentName(
            Context context, ComponentName component, int position) {

        Intent intent = new Intent();
        intent.setComponent(component);

        PackageManager pm = context.getPackageManager();
        ResolveInfo rInfo = pm.resolveActivity(intent, 0);

        if (rInfo == null) {
            return null;
        }

        LauncherActivityInfoCompat info =
                LauncherActivityInfoCompat.fromResolveInfo(rInfo, context);

        IconCache iconCache = LauncherAppState.getInstance().getIconCache();

        AppInfo appInfo = new AppInfo(context, info, info.getUser(), iconCache);
        appInfo.screenId = indexToScreenId(position);
        appInfo.cellX = position;
        appInfo.cellY = 0;
        appInfo.rank = position;
        appInfo.container = LauncherSettings.Favorites.CONTAINER_HOTSEAT;

        return appInfo.makeShortcut();
    }

    private static int indexToScreenId(int index) {
        return index < 2 ? index : index + 1;
    }

    private static void reloadWorkspace() {
        LauncherAppState app = LauncherAppState.getInstanceNoCreate();
        if (app != null) {
            app.reloadWorkspace();
        }
    }

    private static void setMigrationDone(Context context) {
        getSharedPrefs(context).edit().putBoolean(FAVORITES_MIGRATED_KEY, true).apply();
        FavoritesStorageHelper.clearLegacySettings(context);
    }

    private static SharedPreferences getSharedPrefs(Context context) {
        return context.getSharedPreferences(LauncherAppState.getSharedPreferencesKey(),
                Context.MODE_PRIVATE);
    }
}
